-------------------------------------------------------------------------------
-- key_unit_tb
-- testbench for the key_unit component
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Libraries
-------------------------------------------------------------------------------
-- standard libraries
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library std;
use std.textio.all;
--use work.std_logic_textio.all;        -- the Synopsys text IO package
use ieee.std_logic_textio.all;          -- compiled into the working directory

entity key_unit_tb is
end key_unit_tb;

architecture behav of key_unit_tb is
-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------
  constant PERIOD             : time                      := 10 ns;  -- used for
  constant des_shift_sequence : std_logic_vector(1 to 16) := "0111111011111100";  -- shift sequence
                                        -- for DES

-------------------------------------------------------------------------------
-- Signals
-------------------------------------------------------------------------------
  signal Clk                :     std_logic                 := '0';
  signal Rst                :     std_logic                 := '0';
  signal permkey_s1         :     std_logic_vector(1 to 56) := (others => '0');
  signal shifttwo_s1        :     std_logic                 := '0';
  signal loaddata_s1        :     std_logic                 := '0';
  signal keysub_s1          :     std_logic_vector(1 to 48) := (others => '0');
  signal expected_keysub_s1 :     std_logic_vector(1 to 48) := (others => '0');
-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
  component key_unit
    port (
      keysub_s1             : out std_logic_vector(1 to 48);
      permkey_s1            : in  std_logic_vector(1 to 56);
      shifttwo_s1           : in  std_logic;
      loaddata_s1           : in  std_logic;
      Clk                   : in  std_logic;
      Rst                   : in  std_logic);
  end component;
begin  -- behav

  Rst <= '1', '0'
         after PERIOD*2;

  clock : process
  begin
    clk <= '0';
    wait for period/2;
    clk <= '1';
    wait for period/2;
  end process;

-------------------------------------------------------------------------------
-- DUT
-------------------------------------------------------------------------------
  DUT : key_unit
    port map (
      keysub_s1   => keysub_s1,
      permkey_s1  => permkey_s1,
      shifttwo_s1 => shifttwo_s1,
      loaddata_s1 => loaddata_s1,
      Clk         => Clk,
      Rst         => Rst);

  stimulus                        : process(Clk)
    file inFile                   : text open read_mode is "../vectors/key_unit.vec";  --text file
    variable inline               : line;
    variable x                    : std_logic_vector(1 to 56) := (others => '0');
    variable y                    : std_logic_vector(1 to 48) := (others => '0');
    variable EOF                  : boolean                   := false;
    variable num_vec              : integer range 0 to 4095   := 0;  -- number of lines in the file
    variable num_errors           : integer range 0 to 4095   := 0;
    variable cnt                  : integer range 0 to 15;
  begin
    if(Clk'event and Clk = '1') then
      if(Rst = '1') then
        x                                                     := (others => '0');
        y                                                     := (others => '0');
        loaddata_s1          <= '0';
        shifttwo_s1          <= '0';
        permkey_s1           <= (others                                  => '0');
        expected_keysub_s1   <= (others                                  => '0');
        cnt                                                   := 0;
      else
        EOF                                                   := endfile(inFile);
        if(EOF) then
          if(num_errors = 0) then
            report "Test Passed!!!";
          else
            report "Test Failed!!!";
          end if;
          report "Summary         : Number of Vectors : "&integer'image(num_vec)
            & ", Number of Errors : " &integer'image(num_errors);
          report "'END of FILE'   : Terminating with 'FAILURE' assert" severity failure;
        else
          if(cnt = 0) then
            loaddata_s1      <= '1';
          else
            loaddata_s1      <= '0';
          end if;
          shifttwo_s1        <= des_shift_sequence(cnt+1);
          readline(inFile, inline);
          hread(inline, x);
          permkey_s1         <= x;
          hread(inline, y);
          expected_keysub_s1 <= y;
          if(not(expected_keysub_s1 = keysub_s1))then
            report "Output Vector MISMATCH" severity warning;
            num_errors:= num_errors +1;
          end if;
          if(cnt = 15) then
            cnt:= 0;
          else
            cnt:= cnt +1;
          end if;
          num_vec:= num_vec +1;
        end if;
      end if;
    end if;
  end process;
end behav;
