-------------------------------------------------------------------------------
-- out_permute_tb
-- testbench for the output permute component
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Libraries
-------------------------------------------------------------------------------
-- standard libraries
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library std;
use std.textio.all;
--use work.std_logic_textio.all;        -- the Synopsys text IO package
use ieee.std_logic_textio.all;          -- compiled into the working directory

entity out_permute_tb is
end out_permute_tb;

architecture behav of out_permute_tb is
-------------------------------------------------------------------------------
-- Signals
-------------------------------------------------------------------------------
  signal in_s1                   :     std_logic_vector(1 to 64) := (others => '0');
  signal out_s1, expected_out_s1 :     std_logic_vector(1 to 64) := (others => '0');
-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
  component out_permute
    port (
      out_s1                     : out std_logic_vector(1 to 64);
      in_s1                      : in  std_logic_vector(1 to 64));
  end component;

begin  -- behav
-------------------------------------------------------------------------------
-- DUT
-------------------------------------------------------------------------------
  DUT : out_permute
    port map (
      in_s1  => in_s1,
      out_s1 => out_s1);

  stimulus              : process
    file inFile         : text open read_mode is "../vectors/out_permute.vec";  --text file
    variable inline     : line;
    variable x          : std_logic_vector(1 to 64) := (others => '0');
    variable EOF        : boolean                   := false;
    variable num_vec    : integer range 0 to 4095   := 0;  -- number of lines in the file
    variable num_errors : integer range 0 to 4095   := 0;
  begin
    wait for 10 ns;

    EOF            := endfile(inFile);
    while (not EOF) loop
      EOF          := endfile(inFile);
      if(EOF) then
        exit;
      end if;
      num_vec      := num_vec + 1;
      readline(inFile, inline);
      hread(inLine, x);
      in_s1           <= x;
      hread(inline, x);
      expected_out_s1 <= x;
      wait for 10 ns;
      if(not(expected_out_s1 = out_s1))then
        report "Output Vector MISMATCH" severity warning;
        num_errors := num_errors +1;
      end if;
    end loop;

    -- stop the test 
    -- failure is going to be used to stop the testbench
    report "Summary         : ";
    if(num_errors = 0) then
      report "Test Passed!!!";
    else
      report "Test Failed!!!";
    end if;
    report "Summary         : Number of Vectors : "&integer'image(num_vec)
      & ", Number of Errors : " &integer'image(num_errors);
    report "'END of FILE'   : Terminating with 'FAILURE' assert" severity failure;


  end process;
end behav;
