library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- This is a "DUMMY DES".  It does not do decryption.
-- Used to test the testbed.  Immediately writes all
-- inputs to the outputs.

entity des is
	PORT
	(
		-- outputs
		DesDout : out std_logic_vector (1 to 64);
		InFIFO_Rd : out std_logic;
		OutFIFO_Wr : out std_logic;
		
		-- inputs
		DesDin : in std_logic_vector (1 to 64);
		DesKey : in std_logic_vector (1 to 64);
		InFIFO_Empty : in std_logic;
		OutFIFO_Full : in std_logic;
		Clk : in std_logic;
		Rst : in std_logic
	);
end des;

architecture behav of des is

	signal rd_r : std_logic; -- registered value of inFIFO_Rd
	signal rd_i : std_logic; -- internal version of port inFIFO_Rd
	
begin

	inFIFO_Rd <= rd_i;

	-- Dummy process
	process (inFIFO_Empty, OutFIFO_Full, rd_r, DesDin) begin
			
		-- Read from input fifo if data exists and there is space in the 
		-- output fifo for the result.
		if (inFIFO_Empty = '0' and OutFIFO_Full = '0') then
			rd_i <= '1';
		else
			rd_i <= '0';
		end if;
		
		-- Write to output fifo if result will be available this cycle
		if (rd_r = '1' and OutFIFO_Full = '0') then
			OutFIFO_Wr <= '1';
		else
			OutFIFO_Wr <= '0';
		end if;
		
		-- dummy process so dataout = datain
		DesDout <= DesDin;
				
	end process;
	
	-- flip-flop with syncronous reset
	process (Clk) begin
		if (rising_edge(Clk)) then
			if (Rst = '1') then
				rd_r <= '0';
			else
				rd_r <= rd_i;
			end if;
		end if;
	end process;
			
			
end behav;
