-- Created: Jan 20, 2009
-- Description: ENSC 350 DES testbed example

library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--*****************************************************************************
-- BEGIN ENTITY
--*****************************************************************************

entity des_testbed is
	port
	(
		-- Input ports
		iCLK_50	: in  std_logic;	-- 50MHz clock
		iKEY : in std_logic_vector(3 downto 0);	-- Pushbuttons
		iSW : in std_logic_vector(17 downto 0);	-- Switches

		-- Inout ports

		-- Output ports
		oLEDR : out std_logic_vector(17 downto 0);	-- Red LEDs
		oLEDG : out std_logic_vector(7 downto 0);	-- Green LEDs
        oLCD_RW : out std_logic;	-- R/W control signal for the LCD
        oLCD_EN : out std_logic;	-- Enable control signal for the LCD
        oLCD_RS : out std_logic;	-- Whether or not you are sending an 
									--instruction or character
        oLCD_ON : out std_logic;	-- used to turn on the LCD
        oLCD_BLON : out std_logic;	-- used to turn on the backlight
        LCD_D : out std_logic_vector(7 downto 0)  	-- sends instructions
													-- or characters
	);
	
end des_testbed;

--*****************************************************************************
-- BEGIN ARCHITECTURE
--*****************************************************************************

architecture behav of des_testbed is

--*****************************************************************************
-- components
--*****************************************************************************

	--******* DES COMPONENT GOES HERE *******--
	component des
		PORT
		(
			Clk : in std_logic;
			Rst : in std_logic;
			InFIFO_Empty : in std_logic;
			OutFIFO_Full : in std_logic;
			InFIFO_Rd : out std_logic;
			OutFIFO_Wr : out std_logic;
			DesKey : in std_logic_vector (1 to 64);
			DesDin : in std_logic_vector (1 to 64);
			DesDout : out std_logic_vector (1 to 64)
		);
	end component;
	--***************************************--
	
	
	--******* expected_ram component ********--
	
	-- This component is a ROM, not a ram like in the name. typo.
	-- ROM contents will be initialized with expected_ram.mif
	-- <TO BE DONE>

	--***************************************--
	
	
	--******* testvector_rom component ******--
	
	-- ROM contents will be initialized with testvector_rom.mif
	-- <TO BE DONE>

	--***************************************--
	
	
	--******* result_fifo component *********--
	
	-- This component is used for both INPUT and OUTPUT FIFO
	-- <TO BE DONE>

	--***************************************--

	
--*****************************************************************************
-- typdefs & constants
--*****************************************************************************
	
	-- States for LCD state machine
	type lcd_state_type is (	lcd_s0, 
								lcd_s1, 
								lcd_s2, 
								lcd_s3, 
								lcd_s4, 
								lcd_s5, 
								lcd_s6, 
								lcd_s7,
								lcd_s8, 
								lcd_s9, 
								lcd_s10, 
								lcd_s11, 
								lcd_s12, 
								lcd_s13, 
								lcd_s14,
								lcd_s15, 
								lcd_s16, 
								lcd_s17, 
								lcd_s18, 
								lcd_s19, 
								lcd_s20, 
								lcd_s21,
								lcd_s22, 
								lcd_s23
							); 
	
	-- For ROM address counters
	constant MIN_COUNT : natural := 0;
	constant MAX_COUNT : natural := 1023;
	
	-- lcd clock divisor = 2*(MAX_CLK_CNT + 1)
	-- Divides 50 MHz clock.  
	constant MIN_CLK_CNT  : natural :=0;
	constant MAX_CLK_CNT : natural :=499999; -- generate a 50Hz clock for LCD
	
--*****************************************************************************
-- signals
--*****************************************************************************
	
	-- System level signals
	signal Clk : std_logic; -- nicer name for the 50MHz clock
	signal rst_i : std_logic; -- internal reset is active high
	
	-- LCD signals
	signal lcd_rs_i : std_logic;
	signal lcd_data_i : std_logic_vector(7 downto 0);
	signal lcd_en_i : std_logic;
	signal lcd_clk : std_logic;
	signal lcd_clk_i : std_logic;
	signal lcd_state : lcd_state_type;
	signal lcd_exp_data : std_logic_vector(63 downto 0);
	signal lcd_act_data : std_logic_vector(63 downto 0);
	signal lcd_address : std_logic_vector(9 downto 0);
	signal display_data : std_logic_vector(63 downto 0);
	signal lcd_dslice : std_logic_vector(7 downto 0);
	
	-- Comparison Logic Signals
	signal do_compare : std_logic;
	signal compare_match : std_logic;
	
	-- DES interface signals
	signal key_s1 : std_logic_vector(1 to 64);
	signal desin_s1 : std_logic_vector(1 to 64);
	signal desout_s1 : std_logic_vector(1 to 64);
	
	-- Expected ROM signals
	signal inc_exp_addr : std_logic;
	signal exp_addr : std_logic_vector(9 downto 0);
	signal exp_dout : std_logic_vector(63 downto 0);
	
	-- Testvector ROM signals
	signal inc_tv_addr : std_logic;
	signal tv_addr : std_logic_vector(9 downto 0);
	signal tv_dout : std_logic_vector(63 downto 0);
	
	-- Input FIFO wires
	signal infifo_din : std_logic_vector(63 downto 0);
	signal infifo_rdreq : std_logic;
	signal infifo_wrreq : std_logic;
	signal infifo_empty : std_logic;
	signal infifo_full : std_logic;
	signal infifo_dout : std_logic_vector(63 downto 0);
	signal infifo_usedw : std_logic_vector(5 downto 0);
	
	-- Output FIFO wires
	signal outfifo_din : std_logic_vector(63 downto 0);
	signal outfifo_rdreq : std_logic;
	signal outfifo_wrreq : std_logic;
	signal outfifo_empty : std_logic;
	signal outfifo_full : std_logic;
	signal outfifo_dout : std_logic_vector(63 downto 0);
	signal outfifo_usedw : std_logic_vector(5 downto 0);
	
	
begin --architecture

--*****************************************************************************
-- output port assignments & misc assignments
--*****************************************************************************

	Clk <= iCLK_50; -- 50 MHz
	lcd_clk <= lcd_clk_i; -- 50 Hz
	
	-- FOR DEBUGGING, you may want to comment above line, and tie clocks to a
	-- pushbutton (see commented lines below).  This will allow you to use LEDs
	-- to show status on the board.
	
	--Clk <= not iKEY(3); -- Tie clock to pushbutton
	--lcd_clk <= Clk;	-- If clock is pushbutton then tie lcd_clk to 
						-- pushbutton as well or you will need to click
						-- millions of times to see lcd working
	
	
	rst_i <= not iKEY(0); -- active high reset generated pushed KEY0
	
	-- LCD signal assignments
	oLCD_BLON <= '1';   -- backlight is always on
	oLCD_ON <= '1';     -- LCD is always on
	oLCD_RW <= '0';     -- always writing to the LCD
    oLCD_RS <= lcd_rs_i;
    LCD_D <= lcd_data_i;
	oLCD_EN <= lcd_clk and lcd_en_i;

	-- LED assignments
	oLEDR(0) <= not compare_match; -- Turns on RED LED if match fails
	oLEDG(0) <= compare_match; -- Turns on Green LED if match successful
	
	oLEDR(11 downto 1) <= (others => '0');
	oLEDR(17 downto 12) <= infifo_usedw; -- assigned to get rid of warning
	
	oLEDG(1 downto 1) <= (others => '0');
	oLEDG(7 downto 2) <= outfifo_usedw; -- assigned to get rid of warning
	
	--desin_s1 <= <TO BE DONE>;
	--<To BE DONE> <= desout_s1;

--*****************************************************************************
-- DES key select
--*****************************************************************************

	-- DES keys

	-- SW(1 downto 0) allows you to select 1 or 4 keys.  Only change these
	-- switch values during reset.
	
	-- For each key, there should be a corresponding expected_ram.mif and
	-- testvector_rom.mif file.  ROMs must be initialized with these .mif
	-- files for successful decryption with a given key.
	
	-- In the demo, you will be given a new key and mif files to compile
	-- and demonstrate your circuit.  The keys below are simply example
	-- keys.  Feel free to make your set of testvectors/key as well.
	
	process (iSW)
	begin
		if (iSW(1) = '0' and iSW(0) = '0') then
			key_s1 <= X"0F1E_2D3C_4B5A_6978"; -- KEY ALPHA
		elsif (iSW(1) = '0' and iSW(0) = '1') then
			key_s1 <= X"21A9_D549_094B_7B03"; -- KEY BETA
		elsif (iSW(1) = '1' and iSW(0) = '0') then
			key_s1 <= X"8610_3AAC_6D82_104B"; -- KEY GAMMA
		else -- iSW(1)='1' and iSW(0)='1'
			key_s1 <= X"0123_4567_89AB_CDEF"; -- KEY DELTA
		end if;
	end process;
	
--*****************************************************************************
-- Logic to read data from testvector_rom to input_fifo_i
--*****************************************************************************
	
	-- <TO BE DONE>
	
--*****************************************************************************
-- Logic to read from out_fifo_i and expected_rom
--*****************************************************************************
	
	-- <TO BE DONE>
	
--*****************************************************************************
-- Comparison Logic
--*****************************************************************************	
	
	-- comparator
	process(Clk)
	begin
		if (rising_edge(Clk)) then
			if (rst_i = '1') then
				compare_match <= '1';
			elsif (do_compare = '1') then
				if (outfifo_dout = exp_dout) then
					compare_match <= '1';
				else
					compare_match <= '0';
				end if;
			else
				compare_match <= compare_match;
			end if;
		end if;
	end process;
	

--*****************************************************************************
-- LCD Logic
--*****************************************************************************

	--LCD clock generation
	process (Clk)
		variable clk_cnt : integer range MIN_CLK_CNT to MAX_CLK_CNT;
	begin
		if (rising_edge(Clk)) then
		
			if (rst_i = '1') then
				clk_cnt := MIN_CLK_CNT;
			elsif (clk_cnt = MAX_CLK_CNT) then
				clk_cnt := MIN_CLK_CNT;
			else
				clk_cnt := clk_cnt + 1;
			end if;
			
			if (rst_i = '1') then
				lcd_clk_i <= '0';
			elsif (clk_cnt = MAX_CLK_CNT) then
				lcd_clk_i <= not lcd_clk_i;
			else
				lcd_clk_i <= lcd_clk_i;
			end if;
			
		end if;
	end process;
	
	
	
	--register to capture expected data, actual data, and address
	process (Clk)
	begin
		if (rising_edge(Clk)) then
			if (rst_i = '1') then
				lcd_exp_data <= (others => '0');
				lcd_act_data <= (others => '0');
				lcd_address <= (others => '0');
			else
				
				if (inc_exp_addr = '1') then
					lcd_exp_data <= exp_dout;
					lcd_address <= exp_addr;
					lcd_act_data <= outfifo_dout;
				else
					lcd_exp_data <= lcd_exp_data;
					lcd_address <= lcd_address;
					lcd_act_data <= lcd_act_data;
				end if;
			
			end if;
		end if;
	end process;
	
	
	-- mux to choose value to write to LCD
	-- "00" = DES output data
	-- "01 = Expected ROM data
	-- others = expected ROM address, zero padded to 64-bits
	process (iSW, lcd_act_data, lcd_exp_data, lcd_address)
	begin
		case (iSW(17 downto 16)) is
			when "00" => display_data <= lcd_act_data;
			when "01" => display_data <= lcd_exp_data;
			when others => display_data <= 	"00" 
											& X"0000000000000" 
											& lcd_address;
		end case;
	end process;
			

	-- LCD State Logic
	process (lcd_clk)
	begin
		if (rising_edge(lcd_clk)) then
			if (rst_i = '1') then
				lcd_state <= lcd_s0;
			else
				case lcd_state is
				
					when lcd_s0 => lcd_state <= lcd_s1;
					when lcd_s1 => lcd_state <= lcd_s2;
					when lcd_s2 => lcd_state <= lcd_s3;
					when lcd_s3 => lcd_state <= lcd_s4;
					when lcd_s4 => lcd_state <= lcd_s5;
					when lcd_s5 => lcd_state <= lcd_s6;
					when lcd_s6 => 
						if (iKEY(2) = '0') then
							lcd_state <= lcd_s7;
						else
							lcd_state <= lcd_s6;
						end if;
					when lcd_s7 => lcd_state <= lcd_s8;
					when lcd_s8 => lcd_state <= lcd_s9;
					when lcd_s9 => lcd_state <= lcd_s10;
					when lcd_s10 => lcd_state <= lcd_s11;
					when lcd_s11 => lcd_state <= lcd_s12;
					when lcd_s12 => lcd_state <= lcd_s13;
					when lcd_s13 => lcd_state <= lcd_s14;
					when lcd_s14 => lcd_state <= lcd_s15;
					when lcd_s15 => lcd_state <= lcd_s16;
					when lcd_s16 => lcd_state <= lcd_s17;
					when lcd_s17 => lcd_state <= lcd_s18;
					when lcd_s18 => lcd_state <= lcd_s19;
					when lcd_s19 => lcd_state <= lcd_s20;
					when lcd_s20 => lcd_state <= lcd_s21;
					when lcd_s21 => lcd_state <= lcd_s22;
					when lcd_s22 => lcd_state <= lcd_s23;
					when lcd_s23 => lcd_state <= lcd_s6;					
					when others => lcd_state <= lcd_s0;
				end case;
			end if;
		end if;
	end process;
	
	-- LCD Output logic
	process (lcd_state, display_data)
	begin

		case lcd_state is
			when lcd_s0 => lcd_dslice <= X"38"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s1 => lcd_dslice <= X"38"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s2 => lcd_dslice <= X"0C"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s3 => lcd_dslice <= X"01"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s4 => lcd_dslice <= X"06"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s5 => lcd_dslice <= X"80"; lcd_rs_i <= '0'; lcd_en_i <= '1';
			when lcd_s6 => 
				lcd_dslice <= X"00"; 
				lcd_rs_i <= '1'; lcd_en_i <= '0';
			when lcd_s7 => 
				lcd_dslice <= X"0" & display_data(63 downto 60); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s8 => 
				lcd_dslice <= X"0" & display_data(59 downto 56); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s9 => 
				lcd_dslice <= X"0" & display_data(55 downto 52); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s10 => 
				lcd_dslice <= X"0" & display_data(51 downto 48); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s11 => 
				lcd_dslice <= X"0" & display_data(47 downto 44); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s12 => 
				lcd_dslice <= X"0" & display_data(43 downto 40); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s13 => 
				lcd_dslice <= X"0" & display_data(39 downto 36); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s14 => 
				lcd_dslice <= X"0" & display_data(35 downto 32); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s15 => 
				lcd_dslice <= X"0" & display_data(31 downto 28); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s16 => 
				lcd_dslice <= X"0" & display_data(27 downto 24); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s17 => 
				lcd_dslice <= X"0" & display_data(23 downto 20); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s18 => 
				lcd_dslice <= X"0" & display_data(19 downto 16); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s19 => 
				lcd_dslice <= X"0" & display_data(15 downto 12); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s20 => 
				lcd_dslice <= X"0" & display_data(11 downto 8); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s21 => 
				lcd_dslice <= X"0" & display_data(7 downto 4); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s22 => 
				lcd_dslice <= X"0" & display_data(3 downto 0); 
				lcd_rs_i <= '1'; lcd_en_i <= '1';
			when lcd_s23 => 
				lcd_dslice <= X"02"; 
				lcd_rs_i <= '0'; lcd_en_i <= '1'; 
			when others => 
				lcd_dslice <= X"00"; 
				lcd_rs_i <= '1'; lcd_en_i <= '0';
		end case;

	end process;

	
	-- lookup table for hex ASCII codes for LCD
	process (lcd_dslice, lcd_rs_i)
	begin
		if (lcd_rs_i = '0') then
			lcd_data_i <= lcd_dslice;
		else
			case (lcd_dslice(3 downto 0)) is
				when X"0" => lcd_data_i <= X"30";
				when X"1" => lcd_data_i <= X"31";
				when X"2" => lcd_data_i <= X"32";
				when X"3" => lcd_data_i <= X"33";
				when X"4" => lcd_data_i <= X"34";
				when X"5" => lcd_data_i <= X"35";
				when X"6" => lcd_data_i <= X"36";
				when X"7" => lcd_data_i <= X"37";
				when X"8" => lcd_data_i <= X"38";
				when X"9" => lcd_data_i <= X"39";
				when X"A" => lcd_data_i <= X"41";
				when X"B" => lcd_data_i <= X"42";
				when X"C" => lcd_data_i <= X"43";
				when X"D" => lcd_data_i <= X"44";
				when X"E" => lcd_data_i <= X"45";
				when X"F" => lcd_data_i <= X"46";
				when others => lcd_data_i <= X"00";
			end case;
		end if;
	end process;

--*****************************************************************************
-- Component Instantiation Statements
--*****************************************************************************

	--******* DES INSTANTIATION GOES HERE *******--

	-- <TO BE DONE>

	--*******************************************--



	--*** expected_ram INSTANTIATION GOES HERE **--

	-- <TO BE DONE>

	--*******************************************--
	
	
	
	--* testvector_rom INSTANTIATION GOES HERE **--

	-- <TO BE DONE>

	--*******************************************--
	
	
	
	--**** input_fifo INSTANTIATION GOES HERE ***--

	-- <TO BE DONE>

	--*******************************************--
	
	
	
	--*** output_fifo INSTANTIATION GOES HERE ***--

	-- <TO BE DONE>

	--*******************************************--
	


--*****************************************************************************

end behav;
